<?php

/**
 * Isotope eCommerce for Contao Open Source CMS
 *
 * Copyright (C) 2009-2016 terminal42 gmbh & Isotope eCommerce Workgroup
 *
 * @link       https://isotopeecommerce.org
 * @license    https://opensource.org/licenses/lgpl-3.0.html
 */

namespace Isotope\Module;

use Isotope\Isotope;
use Isotope\Model\ProductType;

/**
 * AbstractProductFilter provides basic methods to handle product filtering
 *
 * @property array  $iso_searchFields
 * @property string $iso_searchAutocomplete
 * @property array  $iso_filterFields
 * @property bool   $iso_filterHideSingle
 * @property string $iso_newFilter
 * @property array  $iso_sortingFields
 * @property string $iso_listingSortField
 * @property string $iso_listingSortDirection
 * @property bool   $iso_enableLimit
 * @property int    $iso_perPage
 * @property string $iso_filterTpl
 */
abstract class AbstractProductFilter extends Module
{
    const FILTER_NEW = 'show_new';
    const FILTER_OLD = 'show_old';

    /**
     * Constructor.
     *
     * @param \ModuleModel|object $objModule
     * @param string $strColumn
     */
    public function __construct($objModule, $strColumn = 'main')
    {
        parent::__construct($objModule, $strColumn);

        \Controller::loadDataContainer('tl_iso_product');
        \System::loadLanguageFile('tl_iso_product');
    }

    /**
     * @inheritdoc
     */
    protected function getSerializedProperties()
    {
        $props = parent::getSerializedProperties();

        $props[] = 'iso_filterFields';
        $props[] = 'iso_sortingFields';
        $props[] = 'iso_searchFields';

        return $props;
    }

    /**
     * Returns an array of attribute values found in the product table
     *
     * @param string $attribute
     * @param array  $categories
     * @param string $newFilter
     * @param string $sqlWhere
     *
     * @return array
     */
    protected function getUsedValuesForAttribute($attribute, array $categories, $newFilter = '', $sqlWhere = '')
    {
        $attributeTypes = $this->getProductTypeIdsByAttribute($attribute);
        $variantTypes   = $this->getProductTypeIdsByAttribute($attribute, true);
        $atypeCount     = count($attributeTypes);
        $vtypeCount     = count($variantTypes);

        if (0 === $atypeCount && 0 === $vtypeCount) {
            return array();
        }

        $values         = array();
        $typeConditions = array();
        $join           = '';
        $categoryWhere  = '';
        $published      = '';
        $time           = \Date::floorToMinute();

        if ('' !== (string) $sqlWhere) {
            $sqlWhere = ' AND ' . (string) $sqlWhere;
        }

        // Apply new/old product filter
        if (self::FILTER_NEW === $newFilter) {
            $sqlWhere .= ' AND p1.dateAdded>=' . Isotope::getConfig()->getNewProductLimit();
        } elseif (self::FILTER_OLD === $newFilter) {
            $sqlWhere .= ' AND p1.dateAdded<' . Isotope::getConfig()->getNewProductLimit();
        }

        if (BE_USER_LOGGED_IN !== true) {
            $published = "
                AND p1.published='1'
                AND (p1.start='' OR p1.start<'$time')
                AND (p1.stop='' OR p1.stop>'" . ($time + 60) . "')
            ";
        }

        if (0 !== $atypeCount) {
            $typeConditions[] = 'p1.type IN (' . implode(',', $attributeTypes) . ')';
        }

        if (0 !== $vtypeCount) {
            $typeConditions[] = 'p2.type IN (' . implode(',', $variantTypes) . ')';
            $join             = 'LEFT OUTER JOIN tl_iso_product p2 ON p1.pid=p2.id';
            $categoryWhere    = 'OR p1.pid IN (
                                    SELECT pid
                                    FROM tl_iso_product_category
                                    WHERE page_id IN (' . implode(',', $categories) . ')
                                )';

            if (BE_USER_LOGGED_IN !== true) {
                $published .= " AND (
                    p1.pid=0 OR (
                        p2.published='1'
                        AND (p2.start='' OR p2.start<'$time')
                        AND (p2.stop='' OR p2.stop>'" . ($time + 60) . "')
                    )
                )";
            }
        }

        $result = \Database::getInstance()->execute("
            SELECT DISTINCT p1.$attribute AS options
            FROM tl_iso_product p1
            $join
            WHERE
                p1.language=''
                AND p1.$attribute!=''
                " . $published . '
                AND (
                    p1.id IN (
                        SELECT pid
                        FROM tl_iso_product_category
                        WHERE page_id IN (' . implode(',', $categories) . ")
                    )
                    $categoryWhere
                )
                AND (
                    " . implode(' OR ', $typeConditions) . "
                )
                $sqlWhere
        ");

        while ($result->next()) {
            if ($this->isCsv($attribute)) {
                $values = array_merge($values, explode(',', $result->options));
            } else {
                $values = array_merge($values, deserialize($result->options, true));
            }
        }

        return array_unique($values);
    }

    /**
     * Get the sorting labels (asc/desc) for an attribute
     *
     * @param string
     *
     * @return array
     */
    protected function getSortingLabels($field)
    {
        $arrData = &$GLOBALS['TL_DCA']['tl_iso_product']['fields'][$field];

        switch ($arrData['eval']['rgxp']) {
            case 'price':
            case 'digit':
                return array($GLOBALS['TL_LANG']['MSC']['low_to_high'], $GLOBALS['TL_LANG']['MSC']['high_to_low']);

            case 'date':
            case 'time':
            case 'datim':
                return array($GLOBALS['TL_LANG']['MSC']['old_to_new'], $GLOBALS['TL_LANG']['MSC']['new_to_old']);
        }

        return array($GLOBALS['TL_LANG']['MSC']['a_to_z'], $GLOBALS['TL_LANG']['MSC']['z_to_a']);
    }

    /**
     * Returns true if the attribute is multiple choice.
     *
     * @param string $attribute
     *
     * @return bool
     */
    protected function isMultiple($attribute)
    {
        return (bool) $GLOBALS['TL_DCA']['tl_iso_product']['fields'][$attribute]['eval']['multiple'];
    }

    /**
     * Returns true if the attribute contains CSV values.
     *
     * @param string $attribute
     *
     * @return bool
     */
    protected function isCsv($attribute)
    {
        return $this->isMultiple($attribute)
            && ',' === $GLOBALS['TL_DCA']['tl_iso_product']['fields'][$attribute]['eval']['csv'];
    }

    /**
     * Get product type IDs with given attribute enabled
     *
     * @param string $attributeName
     * @param bool   $forVariants
     *
     * @return array
     */
    private function getProductTypeIdsByAttribute($attributeName, $forVariants = false)
    {
        static $cache;

        if (null === $cache) {
            /** @var ProductType[] $productTypes */
            $productTypes = ProductType::findAll();
            $cache        = array();

            if (null !== $productTypes) {
                foreach ($productTypes as $type) {
                    foreach ($type->attributes as $attribute => $config) {
                        if ($config['enabled']) {
                            $cache['attributes'][$attribute][] = $type->id;
                        }
                    }

                    if ($type->variants) {
                        foreach ($type->variant_attributes as $attribute => $config) {
                            if ($config['enabled']) {
                                $cache['variant_attributes'][$attribute][] = $type->id;
                            }
                        }
                    }
                }
            }

            foreach ($cache as $property => $attributes) {
                foreach ($attributes as $attribute => $values) {
                    $cache[$property][$attribute] = array_unique($values);
                }
            }
        }

        if ($forVariants) {
            return (array) $cache['variant_attributes'][$attributeName];
        } else {
            return (array) $cache['attributes'][$attributeName];
        }
    }
}
